#include "p16F18323.inc"

; CONFIG1
; __config 0x17FA
 __CONFIG _CONFIG1, _FEXTOSC_HS & _RSTOSC_EXT1X & _CLKOUTEN_OFF & _CSWEN_OFF & _FCMEN_OFF
; CONFIG2
; __config 0x3732
 __CONFIG _CONFIG2, _MCLRE_OFF & _PWRTE_OFF & _WDTE_ON & _LPBOREN_OFF & _BOREN_OFF & _BORV_LOW & _PPS1WAY_OFF & _STVREN_ON & _DEBUG_OFF
; CONFIG3
; __config 0x1FFF
 __CONFIG _CONFIG3, _WRT_BOOT & _LVP_OFF
; CONFIG4
; __config 0x3FFF
 __CONFIG _CONFIG4, _CP_OFF & _CPD_OFF

                                ;********************
                                ;Still to do: AARRRRGGHH!!!!!!!11111one
				;Add clean WDT restart
				;Timezone corr not working
				;Clock freezes if first LED byte altered from 00 - clock side?
                                ;********************
;PIN	1	vdd
;PIN	2	RA5	DI	Xtal1
;PIN	3	RA4	DI	Xtal2
;PIN	4	RA3	DI	IR receivers
;PIN	5	RC5	DI	RXD from PSU
;PIN	6	RC4	AI	Current sense	
;PIN	7	RC3	D0	Coil PWM

;PIN	8	RC2	DO	IR LED PWM
;PIN	9	RC1	DI/DO	Voltage sense/LED3 ground
;PIN	10	RC0	DO	LED2 ground/SCL
;PIN	11	RA2	DO	TXD to LED
;PIN	12	RA1	D0	RGB LED data/SDA
;PIN	13	RA0	DO	Buzzer
;PIN	14	Vss

;Test strings:
		;$GPRMC,232224,,,,,,,,171116,A*0c - GPS in

		;01 01 - Movement sensor activated
;
;02,xx,RGB,02 - LED xx RGB value - $PSU,2,4,G,6*** - $LED9,B,5***
;
;03,x,y,03 - Modify packetbit x to y (0 or 1 mod1) - $PSU,3,9,1*** - $BITx,y***
;
;04,Profile,04 - select profile 0-3 - $PSU,4,3*** - $PROx*** 
;
;06,01,HH,MM,SS,DD,MM,YY,06 - Set time (overwritten by GPS)
;
;06,02,DimHHon,DimMMon,DimHHoff,DimMMoff,DimValue,06 - Set dim on/off times - $PSU,6,2,23,23,23,26,5,6*** - $DIM23,23,23,26,5***
;
;06,03,AlaHH,AlaMM,,,,06 - Set alarm and switch on - $PSU,6,3,23,28,6*** - $ALA23,25***
;
;06,04,Offset,,,,,06 - Set timezone offset - $PSU,6,4,20,***
;
;07,Relay1,Relay2,Relay3,Relay4,Relay5,Relay6,Holdtime,07 - Send Relay bytes to display - $PSU,7,1,2,3,4,5,99,6***
;
;08,cat1cat2,cat3cat4,cat5cat6,cat7cat8,cat9cat0,,08 - Cathode remap - $PSU,8,98,76,54,32,10***
;
;09,EPM,,,,,*xx EEPROM image incoming
;	
;-----------------------------                                
;I/O data from OS for user program found from 20-7fh: 53
;-----------------------------
					
#DEFINE		PacketFlag	PacketD1-0x20
#DEFINE		LEDbuffer	0x50	;LED RGB bits 0x50-58
				
				;Memory organisation:
				;0x2000-201f - PRAM 32 byte buffer - only used when eeprom is updated
				;0x2020-202f - IR data packet to flock
				;0x2030-2038 - Internal LED data
				;0x2039-204f - Variables (70-7f not included)
				;0x2050-207f - Serial receive buffer (48 bytes)
				;0x2080-208f - LED settings 1
				;0x2090-209f - LED settings 2
				;0x20a0-20af - LED settings 3
				;0x20b0-20bf - LED settings 4
				;0x20c0-20cf - LED settings 5
				;0x20d0-20df - LED settings 6
				;0x20e0-20ef - LED settings 7
				;0x20f0-20ff - LED settings 8
							
GMink			equ	0x20	;Minutes			
GHouk			equ	0x21	;Hours in 24h
GDayk			equ	0x22	;Days
GMonk			equ	0x23	;Months
GYeak			equ	0x24	;Years
GSeck			equ	0x25	;Seconds
Ahouk			equ	0x26	;Alarm Hours in 24h
Amink			equ	0x27	;Alarm Minutes
HDimOn			equ	0x28	;Dim display hours on
MDimOn			equ	0x29	;Dim display minutes on
HDimOff			equ	0x2a	;Dim display hours off
MDimOff			equ	0x2b	;Dim display minutes off			
Buzzcount		equ     0x2c    ;Counter for buzzer cycles
BuzzNote		equ     0x2d    ;Buzzer frequency
BuzzLen			equ	0x2e	;Buzzer length of time on
RelayD1			equ	0x2f	;Digits 1 Relay bytes received
RelayD2			equ	0x30	;Digits 2 Relay bytes received		
RelayD3			equ	0x31	;Digits 3 Relay bytes received
RelayD4			equ	0x32	;Digits 4 Relay bytes received
RelayD5			equ	0x33	;Digits 5 Relay bytes received		
RelayD6			equ	0x34	;Digits 6 Relay bytes received
ProfileByte		equ	0x35	;LED/settings profile selected 0-3 current
Movecount		equ	0x36	;Countdown to no movement, loaded from PSU packet
Relaycount		equ	0x37	;Countdown for relay display, loaded from relay packet
ADCCPU			equ	0x38	;ADC CPU temperature updated every second
ADCLEV			equ	0x39	;ADC Maglev Current updated every second
ButtonZ			equ	0x3a	;IR buttons working reg
ButtonX			equ	0x3b	;IR buttons working reg
Dispcount		equ	0x3c	;Countdown for seconds/date display
TempAD			equ	0x3d	;Raw temp ADC reading
CurrAD			equ	0x3e	;Raw Current sense reading
Tcount			equ	0x3f	;Receive byte timeout count for eeprom mode
		
PacketD1		equ	0x40	;Digits 1 (Left-right) DO NOT MOVE
PacketD2		equ	0x41	;Digits 2 (Left-right) DO NOT MOVE		
PacketD3		equ	0x42	;Digits 3 (Left-right) DO NOT MOVE
PacketD4		equ	0x43	;Digits 4 (Left-right) DO NOT MOVE
PacketD5		equ	0x44	;Digits 5 (Left-right) DO NOT MOVE		
PacketD6		equ	0x45	;Digits 6 (Left-right) DO NOT MOVE		
Catty0			equ	0x46	;Cathode 1 remap    DO NOT MOVE
Catty1			equ	0x47	;Cathode 2 remap    DO NOT MOVE		
Catty2			equ	0x48	;Cathode 3 remap    DO NOT MOVE
Catty3			equ	0x49	;Cathode 4 remap    DO NOT MOVE
Catty4			equ	0x4a	;Cathode 5 remap    DO NOT MOVE
Catty5			equ	0x4b	;Cathode 6 remap   DO NOT MOVE    			
Catty6			equ	0x4c	;Cathode 7 remap   DO NOT MOVE
Catty7			equ	0x4d	;Cathode 8 remap   DO NOT MOVE			
Catty8			equ	0x4e	;Cathode 9 remap   DO NOT MOVE
Catty9			equ	0x4f	;Cathode 0 remap   DO NOT MOVE			
		
LEDRG			equ	0x50	;LED right G			
LEDRR			equ	0x51	;LED right R
LEDRB			equ	0x52	;LED right B				
LEDMG			equ	0x53	;LED Mid G
LEDMR			equ	0x54	;LED Mid R
LEDMB			equ	0x55	;LED Mid B				
LEDLG			equ	0x56	;LED Left G
LEDLR			equ	0x57	;LED Left R
LEDLB			equ	0x58	;LED Left B
				
VerK			equ	0x59	;Unlock for IIC writes, must contain 'K'			
EEID			equ	0x5a	;eeprom ID byte
EEID1			equ	0x5b	;eeprom ID byte ASCII 1
EEID2			equ	0x5c	;eeprom ID byte ASCII 2
CalcsumL		equ	0x5d	;Calculated EEPROM checksum LSB
CalcsumH		equ	0x5e	;Calculated EEPROM checksum MSB
EEsumL			equ	0x5f	;Retrieved EEPROM checksum LSB
			
EEsumH			equ	0x60	;Retrieved EEPROM checksum MSB			
Timetemp2		equ	0x61	;Temp variable for timezone corr routine
Timetemp3		equ	0x62	;Temp variable for timezone corr routine
Tick			equ	0x63	;1/100 sec tick
Seck			equ	0x64	;Seconds
Mink			equ	0x65	;Minutes			
Houk			equ	0x66	;Hours in 24h
Dayk			equ	0x67	;Days
Monk			equ	0x68	;Months
Yeak			equ	0x69	;Years
Houk12			equ	0x6a	;Hours 12h
DVAR			equ	0x6b	;Delay counter
DVAR2			equ	0x6c	;Delay counter
Zonecorr		equ	0x6d	;Timezone offset 0-23 to add/subtract from GPS GMT
CKsum			equ	0x6e	;Checksum for packet sent			
VerO			equ	0x6f	;Unlock for IIC writes, must contain 'O'
	
UTL			equ	0x70	;IIC address pointer LSB
UTM			equ	0x71	;IIC pointer MSB
GPSCH			equ	0x72	;checksum MSN
GPSCL			equ	0x73	;checksum LSN
GPSpoint		equ	0x74	;Pointer for GPS data buffer

GPStemp1		equ	0x75	;temp buffer for GPS data
IICC			equ	0x76	;temp buffer for GPS data
		
IICtemp			equ	0x77	;temp data holder for IIC
			

HexAscMSN		equ     0x78    ;Least significant digit to display
HexAscLSN		equ     0x79    ;Most significant digit to display
HexAscIn		equ     0x7a    ;Byte to be BCD'd

PacketBits1		equ 	0x7b	;Sent to flock, display mode control flags
					;Bit7 - Show Seconds - ignored on 6 digit displays
					;Bit6 - Show Date - use date colon/indicators
					;Bit5 - Show Time - Use internal timebase as backup
					;Bit4 - Dim display on flock
					;Bit3 - Fade transition off
					;Bit2 - Ripple on
					;Bit1 - Ripple direction
					;Bit0 - Ripple speed
					
PacketBits2		equ 	0x7c	;Not sent to flock, base control flags
;T					;Bit7 - Display off unswitched
;T					;Bit6 - Display off switched movement sensor
;T					;Bit5 - TOGGLE SUMMERTIME HOUR - adds or removes one hour
					;Bit4 - D/M or M/D TOGGLE
;T					;Bit3 - Sticky date/time mode
;T					;Bit2 - Alarm set
					;Bit1 - Seconds tick
					;Bit0 - 12/24 hour mode
		
GPStemp			equ	0x7d	;checksum LSN	

MENUflags2		equ	0x7e	;Bit 7 - overtemp detected***
					;Bit 6 - overcurrent detected***
					;Bit 5 - 
					;Bit 4 - Set to send profile # to displays
					;Bit 3 - Button 2 detected
					;Bit 2 - Set to send EEID to displays (eeprom update)
					;Bit 1 - Button 1 detected
					;Bit 0 - Power loss detected***
								
INTflags		equ	0x7f	;Bit 7 - GPS data string ready
					;Bit 6 - GPS string being received
					;Bit 5 - Set for PM
					;Bit 4 - Alarm triggered!!!1
					;Bit 3 - Set every second
					;Bit 2 - EEPROM receive mode activated
					;Bit 1 - Flag serial byte received
					;Bit 0 - Triggers IR packet to flock 1 per second

;#########################################################################################################

#define		FMax		0x24			;SMPS Frequency was 3b
#define		FMin		0x11			;Minimum delay value = highest frequency

;#########################################################################################################
;								START OF MAIN CODE
;#########################################################################################################

		org 0x00	;Code here is permanent - used for bootstrap loaders etc.
		goto	0x10			;Reset vector
		nop

	dt  "BA"	    ;EEPROM ID PS CL BA

		org 0x04
		btfsc	PIR1,0
		goto	Timr1			;Timer 0 Interrupt - RTC		
		btfsc	PIR1,5
		goto	GPSRXD			;RXD from IR/GPS
		
		retfie

;****************************************************************************************************
;								
;****************************************************************************************************

		org 0x10	;THIS CANNOT MOVE WILL BREAK EEPROM BACKUP

;Initialise registers

Inity
		clrwdt
		movlw	0x20
		movwf	FSR0H
		clrf	FSR0L
Init1
		clrf	INDF0
		incfsz	FSR0L,f
		goto	Init1
		
		BANKSEL	PORTA
		clrf	PORTA
		clrf	PORTC
		movlw	b'00000000'
		movwf	T0CON0			;Timer 0 1:16 postscaler
		movlw	b'01000010'
		movwf	T0CON1			;Timer 0 1:256 prescaler
;Set I/O pins
		BANKSEL	OSCCON1
		movlw	b'00000000'		;14.7456mhz
		movwf	OSCCON1

		BANKSEL	TRISA
		movlw	b'00111000'
		movwf	TRISA
		movlw	b'00110000'
		movwf	TRISC
		movlw	b'00000001'
		movwf	PIE1			;Enable timer 1 interrupts
		
		BANKSEL	RXPPS
		movlw	b'00010101'		;Set RC5 as serial RXD
		movwf	RXPPS
		BANKSEL	RA1PPS
		movlw	b'00000010'		;Set RC3 as PWM5 - Coil
		movwf	RC3PPS
		movlw	b'00011101'		;Set RC2 as NCO - IR LED
		movwf	RC2PPS
		movlw	b'00010100'		;Set RA2 as TXD
		movwf	RA2PPS
		movlw	b'00011100'		;Set RA0 as timer 0
		movwf	RA0PPS
		
		BANKSEL	WPUA
		clrf	WPUA			;no pullups	
		clrf	WPUC

		BANKSEL	ANSELA
		clrf	ANSELA			;Dammit DO NOT FORGET this bug again!!!!11one
		movlw	b'00010000'
		movwf	ANSELC
		BANKSEL	ADCON0
		movlw	b'01100011'
		movwf	ADCON1
		movlw	b'00000001'		;ADC enable
		movwf	ADCON0

		BANKSEL	FVRCON
		movlw	b'11110011'		;FVR set to 4v for DAC and 1v for ADC
		movwf	FVRCON
		
;Setup PWM5
		BANKSEL	PR2
		movlw	FMax			;start freq.
		movwf	PR2
		BANKSEL	PWM5CON
		bsf	PWM5CON,7		;autorestart
		movlw	FMin
		movwf	PWM5DCH			;PWM1 ratio (set to half PR2)
		BANKSEL	T2CON
		bsf	T2CON,2		
		BANKSEL	SLRCONC
		bcf	SLRCONC,3
	
;Setup NCO
		BANKSEL	NCO1CON			;Things we have learned:
						;NCO registers must be written in this order
						;or they will be ignored.
		movlw	b'00000001'
		movwf	NCO1CLK		
		movlw	b'10000000'
		movwf	NCO1CON		
		movlw	0x15
		movwf	NCO1INCH
		movlw	0x1c
		movwf	NCO1INCL		;Divide ratio 14.7456mhz/38khz/4/2			
		BANKSEL	PORTA
		
;Setup timer 1 for RTC
		movlw	b'00100001'
		movwf	T1CON			;Set timer 1 /4 prescaler

;Setup RXD for GPS data
		BANKSEL	BAUD1CON
		movlw	b'00001000'	
		movwf	BAUD1CON		;Set 16 bit BRG
		movlw	.0
		movwf	SP1BRGL	
		movlw	.3
		movwf	SP1BRGH			;Set to 1200bps

		movlw	b'10010000'		;Enable serial, RXD
		movwf	RC1STA
		movlw	b'00100000'		;Enable serial, TXD
		movwf	TX1STA
		BANKSEL	PIE1
		bsf	PIE1,5			;Enable RXD interrupts

;Set start conditions
		BANKSEL	PORTA
		movlw	0xff
		movwf	TMR1H
		clrf	TMR1L
		
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		movlw	.01
		movwf	Seck			;Init time registers
		;movlw	.08
		movwf	Mink
		;movlw	.01
		movwf	Houk
		movwf	Houk12
		;movlw	.04
		movwf	Dayk			;Init time registers
		;movlw	.03
		movwf	Monk
		;movlw	.17
		movwf	Yeak
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		clrf	Catty0			;init cat remaps
		;movlw	.1
		movwf	Catty1
		movlw	.2
		movwf	Catty2
		movlw	.3
		movwf	Catty3
		movlw	.4
		movwf	Catty4
		movlw	.5
		movwf	Catty5
		movlw	.6
		movwf	Catty6
		movlw	.7
		movwf	Catty7
		movlw	.8
		movwf	Catty8
		movlw	.9
		movwf	Catty9
		;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
		movlw	b'00000010'		;Set IR LED grounds to high
		movwf	PORTA
		movlw	b'00000011'
		movwf	PORTC		
		
		bsf	INTCON,7
		bsf	INTCON,6
		
		BANKSEL	PORTA
		call	longdel	
		call	longdel	
		call	LEDon		
		call	longdel
		
		movlw	0x20
		movwf	FSR1H
		movlw	0x8c
		movwf	FSR1L
		
		movlw	0x20
		movwf	PacketBits1		;Select display bits in profiles 0-3
		movwf	INDF1
		bsf	FSR1L,5
		movwf	INDF1
		bsf	FSR1L,6
		movwf	INDF1
		bcf	FSR1L,5
		movwf	INDF1
		
		movlw	0x9e
		movwf	FSR1L
		
		movlw	0x08
		movwf	INDF1			;Set dimmed brightness in profiles 0-3
		bsf	FSR1L,5
		movwf	INDF1
		bsf	FSR1L,6
		movwf	INDF1
		bcf	FSR1L,5
		movwf	INDF1
		
		movlw	0x01
		movwf	PacketBits2		;Select control bits
		
		clrf	MENUflags2
		clrf	INTflags
		
		call	LEDon
		call	longdel
		goto	start
		
;--------------------------------------------------------------------------------------------------------
monthcode	;Lookup table for days in each month - run to get corret value on month rollover. Timetemp2 = month, 3 = year
;--------------------------------------------------------------------------------------------------------

		BANKSEL	PORTA
		movf	Timetemp2,w	
		movlw	0x02			;month Feb?
		xorwf	Timetemp2,w
		btfsc	STATUS,Z
		goto	leapyear		;Yes, goto checkyear
leapback
		movf	Timetemp2,w	
monthlut
		brw				;WARNING!!! make sure it doesn't cross page boundary
		retlw	.30			;Feb 29 leap years
		retlw	.32			;Jan
		retlw	.29
		retlw	.32
		retlw	.31
		retlw	.32
		retlw	.31
		retlw	.32
		retlw	.32
		retlw	.31
		retlw	.32
		retlw	.31 
		retlw	.32			;Dec

leapyear
		btfsc	Timetemp3,0
		goto	leapback
		btfsc	Timetemp3,1
		goto	leapback
		movlw	0x00			;Both bits 0, it's a leap year
		goto	monthlut
		
;--------------------------------------------------------------------------------------------------------
IICread					;
;--------------------------------------------------------------------------------------------------------

		bcf	INTCON,7
		call	IICreadRaw
		bsf	INTCON,7
		return
IICreadRaw	
		BANKSEL	PORTA
		call	InitIIC			;Setup bus for atart, disable INTS
		movlw	b'10100000'		;24c64 address, read (bit 0 clear)
		call	IICout
		call	NACK			;Send ACK
		movf	UTM,w			;Address MSB
		call	IICout
		call	NACK			;Send ACK
		movf	UTL,w			;Address LSB
		call	IICout
		call	NACK			;Send ACK	
		call	NRESTART
		
		movlw	b'10100001'		;24c64 address, read (bit 0 clear)
		call	IICout
		call	NACK			;Send ACK
		call	NIN
		call	IICinin			;read byte
		movwf	IICC
		call	NSTOP			;Set STOP
		goto	IICexit			;Restore interrupts

;--------------------------------------------------------------------------------------------------------
IICwrite				;
;--------------------------------------------------------------------------------------------------------

		bcf	INTCON,7
		call	IICwriteRaw
		bsf	INTCON,7
		return
IICwriteRaw
		BANKSEL	PORTA
		movlw	"O"			;Check write unlock
		xorwf	VerO,w
		btfss	STATUS,Z
		return
		
		movlw	"K"			;Check write unlock
		xorwf	VerK,w
		btfss	STATUS,Z
		return
	
		call	InitIIC			;Setup bus for atart, disable INTS
		movlw	b'10100000'		;24c64 address, read (bit 0 clear)
		call	IICout
		call	NACK			;Send ACK
		movf	UTM,w			;Address MSB
		call	IICout
		call	NACK			;Send ACK
		movf	UTL,w			;Address LSB
		call	IICout
		call	NACK			;Send ACK
		
		movf	IICC,w
		call	IICout			;Send byte in IICC to EEPROM	
		call	NACK			;Send ACK	
		call	NSTOP			;Set STOP
		clrf	VerO
		clrf	VerK
		
		movlw	.21
		movwf	DVAR2
		call	d0			;EEPROM Write delay
		
		goto	IICexit			;Restore interrupts

;--------------------------------------------------------------------------------------------------------
						;IIC IO routines
;--------------------------------------------------------------------------------------------------------

lildel
		nop
		nop
		nop
		return
InitIIC
		BANKSEL	TRISA
		bcf	PORTC,0			;SCL out
		BANKSEL	PORTA			;Setup bus start
		bsf	PORTA,1			;SDA high
		call	lildel
		bsf	PORTC,0			;SCL high
		BANKSEL	TRISA
		bcf	PORTA,1			;SDA out
		bcf	PORTC,0			;SCL out
		BANKSEL	PORTA

		bcf	PORTA,1			;SDA low
		call	lildel
		bcf	PORTC,0			;SCL low
		return
IICout						;Send byte in W to IIC
		movwf	IICtemp
		movlw	.8
		movwf	DVAR
IICoutout

		btfsc	IICtemp,7
		goto	IICoutout1	
		bcf	PORTA,1
		goto	IICoutout2
IICoutout1
		bsf	PORTA,1
IICoutout2
		call	lildel
		bsf	PORTC,0			;SCL high
		call	lildel
		bcf	PORTC,0			;SCL low
		call	lildel
		lslf	IICtemp,f	
		decfsz	DVAR,f
		goto	IICoutout
		bcf	PORTA,1			;SDA low
		return
IICinin						;read byte from IIC, return in IICtemp and W
		BANKSEL	PORTA
		movlw	.8
		movwf	DVAR
IICininin
		lslf	IICtemp,f
		bsf	PORTC,0			;SCL high
		call	lildel
		bsf	IICtemp,0
		btfss	PORTA,1	
		bcf	IICtemp,0
		bcf	PORTC,0			;SCL low
		call	lildel
		decfsz	DVAR,f
		goto	IICininin
		bsf	PORTA,1			;SDA high

		BANKSEL	TRISA
		bcf	PORTA,1			;SDA out
		BANKSEL	PORTA
		movf	IICtemp,w		;Move byte to W
		return
NACK	
		bcf	PORTA,1			;SDA low
		call	lildel
		bsf	PORTC,0			;SCL high
		call	lildel
		bcf	PORTC,0			;SCL low
		call	lildel
		return
NIN						;Set for data in
		BANKSEL	TRISA
		bsf	TRISA,1
		BANKSEL	PORTA
		return
NSTOP						;Set stop condition
		bcf	PORTA,1			;SDA low
		call	lildel
		bsf	PORTC,0			;SCL high
		call	lildel
		bsf	PORTA,1			;SDA high
		call	lildel
		return
NRESTART
		bsf	PORTA,1			;SDA high
		call	lildel
		bsf	PORTC,0			;SCL high
		call	lildel
		bcf	PORTA,1			;SDA low
		call	lildel
		bcf	PORTC,0			;SCL low
		return		
IICexit
		BANKSEL	TRISA
		bsf	PORTC,0			;SCL restored to in
		BANKSEL	PORTA	
		return
		
;--------------------------------------------------------------------------------------------------------		
EEnewrestore					;Copy new eprom into PRAM and reboot
;--------------------------------------------------------------------------------------------------------				
		
		bcf	INTflags,2
		bcf	INTCON,7
		movlw	0x04
		movwf	UTM			;Point to LOW eeprom image, 0200-
		clrf	UTL
EEnewrestore1	
		clrf	FSR0L
		call	PRAMwipe
		call	PRAMwrite		;Write 32 bytes to program eeprom

		incf	UTL,f
		decfsz	UTL,f
		goto	EEnewrestore1
		incf	UTM
		movlw	0x10			
		xorwf	UTM,w
		btfss	STATUS,Z		;0x10?
		goto	EEnewrestore1
		reset
		
;--------------------------------------------------------------------------------------------------------
		
PRAMwrite					;Write block of 32 bytes (16 words) at INDF1
		BANKSEL	NVMADRL
		lsrf	UTM,w
		movwf	NVMADRH
		rrf	UTL,w
		movwf	NVMADRL

		movlw	0x20
		movwf	FSR0H

		clrf	FSR0L			;Set eeprom 64 byte counter
		bcf	NVMCON1,NVMREGS
		bsf	NVMCON1,WREN
		bsf	NVMCON1,LWLO
PRAMwrite1	
		BANKSEL	PORTA
		call	IICreadRaw
		movf	IICC,w
		BANKSEL	NVMADRL
		movwf	NVMDATH
		incf	UTL,f
		incf	FSR0L,f
		BANKSEL	PORTA
		call	IICreadRaw
		movf	IICC,w
		BANKSEL	NVMADRL
		movwf	NVMDATL
		incf	UTL,f
		incf	FSR0L,f
		btfsc	FSR0L,6
		goto	PRAMwrite2
		call	PRAMen
		incf	NVMADRL,f
		goto	PRAMwrite1
			
PRAMwrite2
		bcf	NVMCON1,LWLO
		call	PRAMen
		bcf	NVMCON1,WREN
		return
			
PRAMwipe					;Erase block at INDF1
		BANKSEL	NVMADRL
		
		lsrf	UTM,w
		movwf	NVMADRH
		rrf	UTL,w
		movwf	NVMADRL
		bsf	NVMADRH,7

		bcf	NVMCON1,NVMREGS
		bsf	NVMCON1,FREE
		bsf	NVMCON1,WREN
		
		call	PRAMen
		bcf	NVMCON1,WREN
		return
		
PRAMen		
		movlw	0x55
		movwf	NVMCON2
		movlw	0xaa
		movwf	NVMCON2
		bsf	NVMCON1,WR
		nop
		nop
		return
		
;--------------------------------------------------------------------------------------------------------
longdel			                        ;Delay loops
;--------------------------------------------------------------------------------------------------------
		
		movlw   0xff	
		movwf	DVAR2
d0
		movlw   0xff
		movwf   DVAR
d1
		clrwdt
		;nop
		;nop
		decfsz  DVAR,F
		goto    d1
		decfsz  DVAR2,F
		goto    d0
		return
		
;--------------------------------------------------------------------------------------------------------		
LEDon							;Send 3x24 bits to LEDs	
;--------------------------------------------------------------------------------------------------------		
		
		movlw   .9				;Setup to send 9 bytes (3 LEDs)
		movwf   DVAR2	
		movlw	LEDbuffer
		movwf	FSR1L
		clrf	FSR1H
d1x		
		movlw   .8
		movwf   DVAR
		movlw	.31				;Maximum brightness limit
		andwf	INDF1,f
d1z
		rlf	INDF1,f
		bcf	PORTA,1			
		btfss	STATUS,C
		goto	LEDoff
		bsf	PORTA,1				;Send '1'
		nop
LEDback
		decfsz  DVAR,F
		goto    d1z
		rlf	INDF1,f
		incf	FSR1L,f
		decfsz  DVAR2,F
		goto    d1x
		bcf	PORTA,1	
		return
LEDoff
		bsf	PORTA,1				;Send '0'
		bcf	PORTA,1
		nop
		goto    LEDback
		
;--------------------------------------------------------------------------------------------------------
Hex2BCD						;Take value in Hex and convert to 2 BCD digits
;--------------------------------------------------------------------------------------------------------

		movwf	HexAscIn
		clrf	HexAscMSN
		movlw	.10
Hex2BCD1
		incf	HexAscMSN,f
		subwf	HexAscIn,f
		btfsc	STATUS,C
		goto	Hex2BCD1
		decf	HexAscMSN,f
		addwf	HexAscIn,w
		movwf	HexAscLSN
		return

;--------------------------------------------------------------------------------------------------------
GPSRXD						;Read incoming RXD byte from GPS
;--------------------------------------------------------------------------------------------------------
					
		BANKSEL	RC1REG
		movf	RC1REG,w
		BANKSEL	PORTA
		bsf	INTflags,1		;Flag byte received
		movwf	GPStemp1
		btfsc	INTflags,2		;Check flag for EEPROM in
		goto	GPSRXDex		;Exit without using if EEPROM receive active
		
		andlw	0x1f
		btfsc	INTflags,7		;Exit if string ready to process
		goto	GPSRXDex
		btfsc	INTflags,6		;bypass if already receiving a string
		goto	GPSRXDin		

		movlw	"$"			;Look for start byte
		xorwf	GPStemp1,w
		btfss	STATUS,Z
		goto	GPSRXDex		;Not start,exit	
		
		movlw	.80
		movwf	GPSpoint		;Setup buffer pointer
		bsf	INTflags,6		;Flag string to receive
		goto	GPSRXDex		;Exit ready to receive on next byte

GPSRXDin
		movlw	0x20
		movwf	FSR0H
		movf	GPSpoint,w
		movwf	FSR0L
		movf	GPStemp1,w
		movwf	INDF0
		incf	GPSpoint,f
		movlw	.128			;Look for start byte
		xorwf	GPSpoint,w
		btfsc	STATUS,Z
		goto	GPSRXDerrr		;No stop before end of buffer, discard data and reset

		decf	FSR0L,f
		decf	FSR0L,f
		movf	INDF0,w
		movwf	GPStemp1
		movlw	"*"			;Look for end byte
		xorwf	GPStemp1,w
		btfss	STATUS,Z
		goto	GPSRXDex;3		;No stop byte, exit.
		bcf	INTflags,6		;Clear data being received flag
		bsf	INTflags,7		;Replace with data ready flag		
		goto	GPSRXDex
GPSRXDerrr					;Buffer pointer overflowed with no * detected
		bcf	INTflags,7
		bcf	INTflags,6

GPSRXDex
		BANKSEL	PORTA
		bcf	PIR1,5
		retfie	
		
;#########################################################################################################
;#########################################################################################################
;#########################################################################################################
;#########################################################################################################
				
	org 0x0200				;Code from here is updateable
	
EEname		
	dt  "$GPEEPROMBAS0106",0xff		;.bin code identifier with version and release numbering
	
Timr1
		goto	Timr1Hi			;Entry points for updateable code section
	
;--------------------------------------------------------------------------------------------------------
start
;--------------------------------------------------------------------------------------------------------
		
		BANKSEL	PORTA
		
		movlw	b'00000010'		;Set IR LED grounds to high
		movwf	PORTA
		movlw	b'00000010'		;set RC0 to high
		movwf	PORTC
start1		
		call	checkk
		goto	start1
		
;--------------------------------------------------------------------------------------------------------
checkk			
;--------------------------------------------------------------------------------------------------------
		
		btfsc	INTflags,0		;Check flag to call display refresh etc
		goto	CallInt0
		btfsc	INTflags,3		;Check flag to call once a second
		goto	Call1
checkkin
		btfsc	INTflags,7		;Check if GPS data received
		call	GPSdecode
		return	
		
;--------------------------------------------------------------------------------------------------------
Timr1Hi							;Timer 1 interrupt Reach here 80x sec
;--------------------------------------------------------------------------------------------------------
							;Reload counter for next interrupt
		bcf	PIR1,0				;clear timer 1 int

		movlw	0xd3				;High byte timer 1
		movwf	TMR1H
		BANKSEL	PORTA
		
		btfsc	INTflags,2			;EEPROM receive mode? if so ignore buttons and ADC tree
		goto	CallIntx
		
		
		lsrf	Tick,w
		movwf	GMink				;Temp variable
		lsrf	GMink,f
		
		movlw	.14
		xorwf	GMink,w
		btfsc	STATUS,Z
		goto	IRTREE1
		
		movlw	.15
		xorwf	GMink,w
		btfsc	STATUS,Z
		goto	IRTREE2
		
		movlw	.16
		xorwf	GMink,w
		btfsc	STATUS,Z
		goto	IRTREE4
		
		movlw	.17
		xorwf	GMink,w
		btfsc	STATUS,Z
		goto	IRTREE5
		
		movlw	.18
		xorwf	GMink,w
		btfsc	STATUS,Z
		goto	IRTREE6
CallIntx	
		btfss	PacketBits2,1
		goto	IRTREE8
		movlw	.79
		xorwf	Tick,w
		btfsc	STATUS,Z
		bsf	T0CON0,7		;Tickk
		
IRTREE8		
		incf	Tick,f
		movlw	.80
		xorwf	Tick,w
		btfss	STATUS,Z
		goto	mainsexit
		clrf	Tick
		
		btfsc	INTflags,2			;EEPROM receive mode? if so ignore buttons and ADC tree
		goto	Timr6v
		
		
		clrf	ButtonZ			;Clear button flags
		clrf	ButtonX			;Clear button flags
		bsf	INTflags,0
		call	Dimmtest1		;Check alarm and dim times
		
		btfss	MENUflags2,0		;Flash and beep if power loss
		goto	CallIntx1
		btfss	Seck,0
		goto	CallIntx11
		bsf	INTflags,4
		clrf	LEDMR
		goto	CallIntx13a;CallIntx1
CallIntx11
		bcf	INTflags,4
		bsf	LEDMR,3
		goto	CallIntx13a
CallIntx1	
		btfss	MENUflags2,7		;Flash and beep if overtemp
		goto	CallIntx12
		btfss	Seck,0
		goto	CallIntx112
		bsf	INTflags,4
		clrf	LEDMB
		goto	CallIntx12
CallIntx112
		bcf	INTflags,4
		bsf	LEDMB,3
		goto	CallIntx13a
CallIntx12		
		btfss	MENUflags2,6		;Flash and beep if overcurrent
		goto	CallIntx13
		btfss	Seck,0
		goto	CallIntx113
		bsf	INTflags,4
		clrf	LEDMG
		goto	CallIntx13
CallIntx113
		bcf	INTflags,4
		bsf	LEDMG,3
		goto	CallIntx13a
CallIntx13		
		clrf	LEDMB
CallIntx13a
		call	LEDon		
		incf	Movecount,f
		decfsz	Movecount,f
		decf	Movecount,f		
		incf	Relaycount,f
		decfsz	Relaycount,f
		decf	Relaycount,f	
Timr6v		
		incf	Tcount,f
		decfsz	Tcount,f		;Check if receive timeout is zero, skip if so
		goto	Timr6b
		goto	CallIntx1a	
Timr6b
		decfsz	Tcount,f		;Check if not zero, skip if so
		goto	CallIntx1a
		bsf	INTCON,7
		bcf	INTflags,2		;Clear eeprom mode - timed out		
		
		
CallIntx1a		
		bsf	INTflags,3
		incf	Seck,f
		movlw	.60
		xorwf	Seck,w
		btfss	STATUS,Z
		goto	mainsexit
		clrf	Seck
increment_min
		incf	Mink,f
		movlw	.60
		xorwf	Mink,w
		btfss	STATUS,Z
		goto	mainsexit
		clrf	Mink
increment_hour					;Every 60 minutes here
		incf	Houk,f
		bcf	INTflags,5		;clear PM bit
		clrf	LEDMG			;LED indicator of PM
		movf	Houk,w			;update 12/24 register
		movwf	Houk12
		movlw	.13
		subwf	Houk12,w
		btfss	STATUS,0
		goto 	increment_hour1
		movwf	Houk12			;Over 12 hours, set 1PM 
		incf	Houk12,f
		bsf	INTflags,5		;Set PM bit
		movlw	.2
		movwf	LEDMG			;LED indicator of PM
		
increment_hour1
		movlw	.24
		xorwf	Houk,w
		btfss	STATUS,Z
		goto	mainsexit
		clrf	Houk
increment_day
		movf	Yeak,w
		movwf	Timetemp3		;temp year for leap year test	
		movf	Monk,w
		movwf	Timetemp2
		call	monthcode		;get days in current month
		incf	Dayk,f
		xorwf	Dayk,w
		btfss	STATUS,Z
		goto	mainsexit
		clrf	Dayk
		incf	Dayk,f
increment_month	
		incf	Monk,f
		movlw	.13
		xorwf	Monk,w
		btfss	STATUS,Z
		goto	mainsexit
		clrf	Monk
		incf	Monk,f
increment_year	
		incf	Yeak,f
		movlw	.99
		xorwf	Yeak,w
		btfss	STATUS,Z
		goto	mainsexit
		clrf	Yeak
mainsexit							
		retfie
	
IRTREE1
		bcf	PORTC,1		    ;Set IR switches off
		bsf	PORTC,0		    ;Set IR switches off
		BANKSEL	TRISA
		bsf	PORTC,1		    ;Set in for voltage sense
		movlw	b'11110101'
		BANKSEL	ADCON0
		movwf	ADCON0		    ;Select ADC channel
		
		BANKSEL	NCO1CON
		movlw	b'00000000'
		movwf	NCO1CON
		nop
		nop
		nop
		nop
		BANKSEL	PORTA
		bcf	MENUflags2,0
		btfss	PORTC,1		    ;Is voltage missing?
		bsf	MENUflags2,0	    ;flag power loss - restore on final version 
		
		BANKSEL	NCO1CON
		movlw	b'10000000'
		movwf	NCO1CON
			
		goto	IRTREXIT		
IRTREE2			
		BANKSEL	TRISA
		bcf	PORTC,1	
		bcf	PORTC,0
		BANKSEL	PORTA
		bcf	PORTC,1		    ;Set IR switch on
		movlw	.3
		movwf	DVAR2
		call	d1
		btfsc	PORTA,3		    ;IR detected?
		incf	ButtonZ		    ;Yes, flag it
		bsf	PORTC,1		    ;Set IR switch off
		BANKSEL	ADCON0
		bsf	ADCON0,1	    ;Trigger ADC conversion
		goto	IRTREXIT
		
IRTREE4		
		btfss	PORTA,3		    ;IR detected?
		goto	IRTREE4a	    ;Shouldn't be, clear ButtonZ
		clrf	ButtonZ
IRTREE4a
		bcf	PORTC,0		    ;Set IR switch on	
		movlw	.3
		movwf	DVAR2
		call	d1
		btfsc	PORTA,3		    ;IR detected?
		incf	ButtonX		    ;Yes, flag it
		bsf	PORTC,0		    ;Set IR switch off
		BANKSEL	ADRESH
		movf	ADRESH,w
		BANKSEL	PORTA
		movwf	TempAD		    ;Save as temp reading
		movlw	b'01010001'	    ;select RC4
		BANKSEL	ADCON0
		movwf	ADCON0		    ;Select ADC channel
		goto	IRTREXIT
		
IRTREE5	
		BANKSEL	ADCON0
		bsf	ADCON0,1	    ;Trigger ADC conversion
		goto	IRTREXIT		
		
IRTREE6		
		btfss	PORTA,3		    ;IR detected?
		goto	IRTREE61	    ;Shouldn't be, clear ButtonX
		clrf	ButtonX
		goto	IRTREE61
IRTREE61	
	
		btfss	ButtonZ,2
		goto	IRTREE62
		bsf	T0CON0,7	    ;beeeeeep
		bsf	MENUflags2,1  
		btfsc	INTflags,4	    ;Is alarm triggered?
		bcf	T0CON0,7
		btfsc	INTflags,4	    ;Is alarm triggered?
		bcf	PacketBits2,2	    ;Clear alarm set
		bcf	INTflags,4	    ;Clear alarm triggered
		
IRTREE62
		btfss	ButtonX,2
		goto	IRTREE621
		bsf	T0CON0,7	    ;beeeeeep
		bsf	MENUflags2,3
		btfsc	INTflags,4	    ;Is alarm triggered?
		bcf	T0CON0,7
		btfsc	INTflags,4	    ;Is alarm triggered?
		bcf	PacketBits2,2	    ;Clear alarm set
		bcf	INTflags,4	    ;Clear alarm triggered
IRTREE621
		BANKSEL	ADRESH
		movf	ADRESH,w
		BANKSEL	PORTA
		movwf	CurrAD		    ;Save as current reading
		
IRTREXIT
		BANKSEL	PORTA
		goto	CallIntx
		
;--------------------------------------------------------------------------------------------------------
CallInt0						;Run 1x/sec
;--------------------------------------------------------------------------------------------------------
		
		bcf	INTflags,0
		BANKSEL	PORTA
		
		movlw	0xbb			;ADJUST AS REQUIRED FOR CORRECT RANGE
		subwf	TempAD,w		;Is it over limit?
		btfsc	STATUS,C
		bsf	MENUflags2,7		;Flag overtemp
		
		movlw	0xb9			;ADJUST AS REQUIRED FOR CORRECT RANGE
		subwf	TempAD,w		;Is it below limit?
		btfss	STATUS,C
		bcf	MENUflags2,7		;Clear overtemp
		
		movlw	0x12			;ADJUST AS REQUIRED FOR CORRECT RANGE
		subwf	CurrAD,w		;Is it over limit?
		btfsc	STATUS,C
		bsf	MENUflags2,6		;Flag overcurrent
		
		movlw	0x0c			;ADJUST AS REQUIRED FOR CORRECT RANGE
		subwf	CurrAD,w		;Is it below limit?
		btfss	STATUS,C
		bcf	MENUflags2,6		;Clear overcurrent

		movlw	0x80			;Now check panic bits and disable PWM if needed
		btfss	MENUflags2,6
		goto	CallInt0a
		movlw	.0			;Panic value for PWM
CallInt0a
		btfss	MENUflags2,7
		goto	CallInt0b
		movlw	.0			;Panic value for PWM
CallInt0b
		btfss	MENUflags2,0		;Check panic flags for power down state
		goto	CallInt0c
		movlw	.0			;Panic value for PWM
CallInt0c	
		btfss	PacketBits2,7		;Now check for selected power off modes
		goto	CallInt0d
		movlw	.0			;Panic value for PWM
CallInt0d		
		btfss	PacketBits2,6		;Now check for selected power off modes
		goto	CallInt0e
		incf	Movecount,f
		decfsz	Movecount,f		;Check if movement detected in last 200 seconds
		goto	CallInt0e
		movlw	.0			;Panic value for PWM	
CallInt0e		
		BANKSEL	PWM5CON
		movwf	PWM5CON			;Start or stop PWM output according to panic bits
		BANKSEL	PORTA
		
		incf	Dispcount,f
		decfsz	Dispcount,f		;Display timer already 0, skip reset
		goto	CallInt1n
		goto	CallInt1v
CallInt1n
		decfsz	Dispcount,f
		goto	CallInt1v		;Display timer not 0, exit
		goto	CallInt1z		;Is now, reset to HH:MM
				
CallInt1v		
		btfss	MENUflags2,1		;Change display button pressed?
		goto	CallInt1a	
		btfss	PacketBits1,5
		goto	CallInt1b
		bcf	PacketBits1,7
		bsf	PacketBits1,6
		bcf	PacketBits1,5
		movlw	.4
		goto	CallInt1aaa		;Set date		
CallInt1b		
		btfss	PacketBits1,6
		goto	CallInt1c
		bcf	PacketBits1,6
		bsf	PacketBits1,7
		bcf	PacketBits1,5
		movlw	.15			;Set seconds
		goto	CallInt1aaa		
CallInt1c		
		btfss	PacketBits1,7
		goto	CallInt1aaa
CallInt1z
		bcf	PacketBits1,6
		bcf	PacketBits1,7
		bsf	PacketBits1,5
CallInt1aaa	
		btfss	PacketBits2,3		;Sticky display set?
		movwf	Dispcount		;Countdown for display to change back to HH:MM
		call	CallSavePB1

CallInt1a	
		btfss	MENUflags2,3		;Change profile button pressed?
		goto	CallInt1m
		incf	ProfileByte,f
		movlw	.3
		andwf	ProfileByte,f
		call	ProfUpdate
		
CallInt1m	
		bcf	T0CON0,7
		btfsc	INTflags,4
		bsf	T0CON0,7		;beeeeeep
		
		
		bcf	MENUflags2,3		;Clear button press
		bcf	MENUflags2,1		;Clear button press

		btfsc	MENUflags2,0		;Just exit if power loss
		goto	Sendapacket
		btfsc	MENUflags2,7		;Just exit if overtemp
		goto	Sendapacket
		btfsc	MENUflags2,6		;Just exit if overcurrent
		goto	Sendapacket

		btfsc	MENUflags2,4
		goto	ProfLoad
		incf	Relaycount,f
		decfsz	Relaycount,f
		goto	LoadRelay
		
		btfsc	PacketBits1,6
		goto	LoadDate
		btfsc	PacketBits1,5
		goto	LoadTime
		btfsc	PacketBits1,7
		goto	LoadTime
		goto	LoadRelay		
		
LoadTime					    ;Move Time to packet registers
		movf	Seck,w
		call	Hex2BCD
		movf	HexAscLSN,w
		movwf	PacketD6
		movf	HexAscMSN,w
		movwf	PacketD5
		movf	Mink,w
		call	Hex2BCD
		movf	HexAscLSN,w
		movwf	PacketD4
		movf	HexAscMSN,w
		movwf	PacketD3
		movf	Houk,w
		btfsc	PacketBits2,0		    ;12/24 display
		movf	Houk12,w
		call	Hex2BCD
		movf	HexAscLSN,w
		movwf	PacketD2
		movf	HexAscMSN,w
		movwf	PacketD1
		goto	CallIntSend
		
LoadDate					    ;Move Date to packet registers
		movf	Yeak,w
		call	Hex2BCD
		movf	HexAscLSN,w
		movwf	PacketD6
		movf	HexAscMSN,w
		movwf	PacketD5
		movf	Monk,w
		btfsc	PacketBits2,4		    ;Swap day and month for US
		movf	Dayk,w
		call	Hex2BCD
		movf	HexAscLSN,w
		movwf	PacketD4
		movf	HexAscMSN,w
		movwf	PacketD3
		movf	Dayk,w
		btfsc	PacketBits2,4		    ;Swap day and month for US
		movf	Monk,w
		call	Hex2BCD
		movf	HexAscLSN,w
		movwf	PacketD2
		movf	HexAscMSN,w
		movwf	PacketD1
		goto	CallIntSend
		
LoadRelay					    ;Move direct bytes to packet registers
		movf	RelayD6,w
		movwf	PacketD6
		movf	RelayD5,w
		movwf	PacketD5
		movf	RelayD4,w
		movwf	PacketD4
		movf	RelayD3,w
		movwf	PacketD3
		movf	RelayD2,w
		movwf	PacketD2
		movf	RelayD1,w
		movwf	PacketD1
CallIntSend	
		goto	Sendapacket
		
ProfLoad
		movf	ProfileByte,w		    ;Show profile#
ProfEELoad
		call	Hex2BCD
		movf	HexAscLSN,w
		movwf	PacketD6
		movwf	PacketD5
		movwf	PacketD4
		movwf	PacketD3
		movwf	PacketD2
		movwf	PacketD1
		bcf	MENUflags2,4
		goto	CallIntSend
		
CallSavePB1
		movlw	0x20
		movwf	FSR1H
		swapf	ProfileByte,w
		movwf	UTL
		lslf	UTL,w
		addlw	0x8c			;Point at 128+(profile*32)+12
		movwf	FSR1L
		movf	PacketBits1,w		;Copy profile control bits
		movwf	INDF1
		return
		
;--------------------------------------------------------------------------------------------------------
Call1						;Run once a sec, from main program loop.
;--------------------------------------------------------------------------------------------------------

		bcf	INTflags,3			;Finished, clear flag until next time
		return

;-------------------------------------------------------------------------------------------------------
Sendapacket				;Send packet to flock 2x per second
;--------------------------------------------------------------------------------------------------------

					;$
					;F
					;D

;Each packet is 19 bytes as follows:
;PacketR1G1		equ 	0x0	;LED 4-bit data 1 Red, 1 Green			
;PacketB1R2		equ 	0x1	;LED 4-bit data 1 Blue, 2 Red	
;PacketG2B2		equ 	0x2	;LED 4-bit data 2 Green, 2 Blue
;PacketR3G3		equ 	0x3	;LED 4-bit data 3 Red, 3 Green
;PacketB3R4		equ 	0x4	;LED 4-bit data 3 Blue, 4 Red
;PacketG4B4		equ 	0x5	;LED 4-bit data 4 Green, 4 Blue
;PacketR5G5		equ 	0x6	;LED 4-bit data 5 Red, 5 Green			
;PacketB5R6		equ 	0x7	;LED 4-bit data 5 Blue, 6 Red			
;PacketG6B6		equ 	0x8	;LED 4-bit data 6 Green, 6 Blue				
;PacketR7G7		equ 	0x9	;LED 4-bit data 7 Red, 7 Green	
;PacketB7R8		equ 	0xa	;LED 4-bit data 7 Blue, 8 Red
;PacketG8B8		equ 	0xb	;LED 4-bit data 8 Green, 8 Blue
		
					;If bits 7,6,5 are 0 just display raw 'relay' data
;PacketBits1		equ 	0xc	;Sent to flock, display mode control flags
					;Bit7 - Show Seconds - ignored on 6 digit displays
					;Bit6 - Show Date - use date colon/indicators
					;Bit5 - Show Time - Use internal timebase as backup
					;Bit4 - Cathode cleaning triggered - REPLACE WITH DIM DISPLAY
					;Bit3 - Fade transition off
					;Bit2 - Ripple on
					;Bit1 - Ripple direction
					;Bit0 - Ripple speed				
;DIMbri					;Dim brightness bits 3-0			
;PacketD1				;Digits 1 (Left-right)
;PacketD2				;Digits 2 (Left-right)		
;PacketD3				;Digits 3 (Left-right)
;PacketD4				;Digits 4 (Left-right)
;PacketD5				;Digits 5 (Left-right)		
;PacketD6				;Digits 6 (Left-right)
					;*
					;CKsumH
					;CKsumL
						
		BANKSEL	PORTA
		movlw	0x20
		movwf	FSR0H
		swapf	ProfileByte,w
		movwf	UTL
		lslf	UTL,w
		addlw	0x80			;Point at 128+(profile*32)

		movwf	FSR0L
		movlw	.12			;12 bytes to send
		movwf	VerO			;Temp loop counter

		movlw	"$"
		call	FirmTB
		movlw	"F"
		call	FirmTB
		movlw	"D"
		call	FirmTB			
		clrf	CKsum		       ;All bytes sent from this point are checksummed	
		
Sendapacketloop1
		movf	INDF0,w
		call	Hex2ASC
		movf	HexAscMSN,w
		call	FirmTB
		movf	HexAscLSN,w
		call	FirmTB
		incf	FSR0L,f
		decfsz	VerO,f
		goto	Sendapacketloop1
	
		movf	INDF0,w			;Control byte		
		call	Hex2ASC
		movf	HexAscMSN,w
		call	FirmTB
		movf	HexAscLSN,w
		call	FirmTB
		
		movlw	0x12			;Offset to 0x1e
		addwf	FSR0L,f
		
		movf	INDF0,w			;Dim brightness		
		call	Hex2ASC
		movf	HexAscLSN,w
		call	FirmTB
	
		;movf	GYeak,w		;TEST
		;call	Hex2ASC
		;movf	HexAscMSN,w
		;call	FirmTB
		;movf	HexAscLSN,w
		;call	FirmTB	
		
		movlw	0x20			;Now point to working memory for PacketD1D2-
		movwf	FSR0L
		movlw	.6			;6 bytes to send
		movwf	VerO			;Temp loop counter	
		
Sendapacketloop2
		movf	INDF0,w
		call	Hex2BCD
		movf	FSR0L,w
		movwf	VerK
		movf	HexAscMSN,w
		addlw	0x26			;Cat remap register offset
		movwf	FSR0L
		movlw	0x30
		addwf	INDF0,w
		call	FirmTB			;Enable for 8-bit data;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		movf	HexAscLSN,w
		addlw	0x26			;Cat remap register offset
		movwf	FSR0L
		movlw	0x30
		addwf	INDF0,w
		call	FirmTB
		movf	VerK,w
		movwf	FSR0L
		incf	FSR0L,f
		decfsz	VerO,f
		goto	Sendapacketloop2
Sendaexit
		movf	CKsum,w
		movwf	HexAscMSN		;Temp holder for checksum
		movlw	"*"
		call	FirmTB
		movf	HexAscMSN,w
		call	Hex2ASC
		movf	HexAscMSN,w
		call	FirmTB
		movf	HexAscLSN,w
		call	FirmTB
		return	
FirmTB
		xorwf	CKsum,f
		BANKSEL	TX1REG			;send byte in W to serial out		
		movwf	TX1REG
		BANKSEL	PORTA

		clrf	DVAR2
d0e
		movlw   0x4f
		movwf   DVAR
d1e
		decfsz  DVAR,F
		goto    d1e
		decfsz  DVAR2,F
		goto    d0e
		return
		
;--------------------------------------------------------------------------------------------------------
Hex2ASC						;Take value in Hex and convert to 2 Ascii digits
;--------------------------------------------------------------------------------------------------------

		movwf	HexAscIn
		andlw	0x0f			;lower nybble
		movwf	HexAscLSN
		swapf	HexAscIn,w
		andlw	0x0f			;higher nybble
		movwf	HexAscMSN
		sublw	.9
		btfsc	STATUS,C
		goto	BASECchk1
		movlw	.55
		goto	BASECchk2
BASECchk1	
		movlw	0x30
BASECchk2		
		addwf	HexAscMSN,f
		movf	HexAscLSN,w
		sublw	.9
		btfsc	STATUS,C
		goto	BASECchk11
		movlw	.55
		goto	BASECchk12
BASECchk11	
		movlw	0x30
BASECchk12		
		addwf	HexAscLSN,f
		return

;--------------------------------------------------------------------------------------------------------
GPSdecode					;Verify GPS data string and extract time/date
;--------------------------------------------------------------------------------------------------------
		
		clrf	GPStemp1		;Set up registers
		movlw	0x20
		movwf	FSR0H
		movlw	.80
		movwf	FSR0L

GPSreri
		movlw	"*"
		xorwf	INDF0,w
		btfsc	STATUS,Z
		goto	GPSrerend		;Last byte found,exit

		movlw	.128			;End of buffer
		xorwf	FSR0L,w
		btfsc	STATUS,Z
		goto	GPSrerend		
GPSreBend
		movf	INDF0,w
		xorwf	GPStemp1,f		;XOR current byte, increment pointer and loop
		incf	FSR0L,f
		goto	GPSreri
GPSrerend
		incf	FSR0L,f
		incf	FSR0L,f			;Increment to LSB of checksum
		movf	INDF0,w
		sublw	0x40			;Subtract 0x40 - will overflow if it's a number
		btfsc	STATUS,C
		goto	GPSmsblo1		;Yes, it's a number
		movlw	.55			;It's a letter, A-F
		subwf	INDF0,w
		goto	GPSrerend1
GPSmsblo1
		movlw	0x30
		subwf	INDF0,w		
GPSrerend1
		andlw	0x0f
		movwf	GPSpoint
		swapf	GPSpoint,f		;Swap to high nibble
		decf	FSR0L,f			;Get MSB
		movf	INDF0,w
		sublw	0x40			;Subtract 0x40 - overflow?
		btfsc	STATUS,C
		goto	GPSmsblo2		;Yes, it's a number
		movlw	.55
		subwf	INDF0,w
		goto	GPSmsblo2a
GPSmsblo2
		movlw	0x30
		subwf	INDF0,w
GPSmsblo2a
		andlw	0x0f
		iorwf	GPSpoint,f		;Add both nibbles
		swapf	GPSpoint,f		;and swap the right way round

		movf	GPSpoint,w		;Compare checksums
		xorwf	GPStemp1,w
		btfss	STATUS,Z

		goto	GPSdecodeXtra		;Bytes don't match, maybe a BT string?

;We have a valid string, now extract the data we need from it
;$GPRMC,092751.000,A,5321.6802,N,00630.3371,W,0.06,31.66,280511,,,A*45		
;$GPRMC,232224,,,,,,,,171116,A*0C
GPSdecodeOK					;Bytes do match, it's GPS data
		movlw	.82;0x02
		movwf	FSR0L
		movlw	"R"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeXtra		;Not the string we want, exit
		incf	FSR0L,f
		movlw	"M"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeXtra		;Not the string we want, exit
		incf	FSR0L,f
		movlw	"C"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeXtra		;Not the string we want, exit
		incf	FSR0L,f			
		
;It's GPS time, load it into the time registers
		BANKSEL	PORTA
		call	GPSgetbyte		;Read time bytes (6 characters)
		movwf	GHouk	
		call	GPSgetbyte
		movwf	GMink			
		call	GPSgetbyte
		movwf	GSeck

		movlw	0x08
		movwf	DVAR		;count ,s
GPSdecodeloop
		incf	FSR0L,f
		movlw	","
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeloop	;Not the comma, looop
		decfsz	DVAR,f
		goto	GPSdecodeloop	;Not the LAST comma, looop

		call	GPSgetbyte		;Read time bytes (6 characters)
		movwf	GDayk	
		call	GPSgetbyte
		movwf	GMonk			
		call	GPSgetbyte
		movwf	GYeak
		;goto	NoDummy

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;DUMMY GPS DATA

	;	clrf	Seck
	;	movlw	.22
	;	movwf	GMink
	;	movlw	.14
	;	movwf	GHouk
	;	movlw	.28
	;	movwf	GDayk
	;	movlw	.2
	;	movwf	GMonk
	;	movlw	.24
	;	movwf	GYeak;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;DUMMY DATA FOR TIMEZONE TEST

	;	movlw	.10
	;	movwf	Zonecorr

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

NoDummy	
		movf	Zonecorr,w
		movwf	Timetemp2
		movlw	.13
		subwf	Timetemp2,f

		btfss	STATUS,C
		goto	CorrectForZone0	;Apply timezone correction (+)
		incf	Timetemp2,f		;Turn 0 -11 into 1-12
		call	CorrectForZone5	;Apply timezone correction (-)
		goto	CorrectForZone99

CorrectForZone0
		call	CorrectForZone
CorrectForZone99
		btfsc	PacketBits2,5;;;;;;;;;;;;;;;;;;;;;;;;;;;
		call	CorrectForBST	;Now add BST hour onto GMT

IRtimeupd				
		movf	GDayk,w
		movwf	Dayk
		movf	GMonk,w
		movwf	Monk
		movf	GYeak,w
		movwf	Yeak
		movf	GHouk,w
		movwf	Houk
		movf	GSeck,w
		movwf	Seck
		movf	GMink,w
		movwf	Mink
		call	VeriTime		;flag time as set

GPSdecodeX					;Exit point
		bcf	INTflags,7		;Clear check if GPS data received
		bsf	INTCON,IOCIE		;enable IOCs for next BT packet
		BANKSEL	PORTA	
		return	
		
GPSgetbyte					;Read 2 BCD ascii bytes and convert to dec 0-99 in hex
		call	IncAndSkip		;Skip any leading ,'s
		movlw	0x30
		subwf	INDF0,w
		andlw	0x0f
		movwf	GPStemp
		
		incf	FSR0L,f
		movlw	","
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSgetbyte1				;Only 1 digit, treat as LSN
		decf	FSR0L,f
		movf	GPStemp,w
		return	
IncAndSkip
		incf	FSR0L,f			
		movlw	","			;Skip any leading ,'s
		xorwf	INDF0,w
		btfsc	STATUS,Z		
		incf	FSR0L,f
		return		
		
GPSgetbyte1		
		movlw	"*"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSgetbyte2				;Only 1 digit, treat as LSN
		decf	FSR0L,f
		movf	GPStemp,w
		return	
GPSgetbyte2		
		movf	GPStemp,w
		lslf	GPStemp,f		;Not a ',' so prepare for next byte to be LSN
		lslf	GPStemp,f
		lslf	GPStemp,f
		addwf	GPStemp,f
		addwf	GPStemp,f
		movlw	0x30
GPSgetnybbledX
		subwf	INDF0,w
		andlw	0x0f
		addwf	GPStemp,f		;Add both nibbles and return in W
		movf	GPStemp,w
		return
		
GPSgetnybbled					;Read 2 ascii bytes 0-9 and convert to hex 00-99
		call	IncAndSkip		;Skip any leading ,'s
		movlw	0x30
		btfss	INDF0,6
		goto	GPSgetnybbledZ
		bcf	INDF0,5
		movlw	.55

GPSgetnybbledZ
		subwf	INDF0,w
		andlw	0x0f
		movwf	GPStemp
		
		incf	FSR0L,f
		movlw	","
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSgetnybbled1		;Only 1 digit, treat as LSN
		decf	FSR0L,f
		movf	GPStemp,w
		return			
GPSgetnybbled1		
		movlw	"*"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSgetnybbled2		;Only 1 digit, treat as LSN
		decf	FSR0L,f
		movf	GPStemp,w
		return	
GPSgetnybbled2	
		swapf	GPStemp,f		;move first nybble to high 4 bits
		movlw	0x30
		btfss	INDF0,6
		goto	GPSgetnybbledX
		bcf	INDF0,5
		movlw	.55
		goto	GPSgetnybbledX
		
;--------------------------------------------------------------------------------------------------------			
GPSdecodeXtra					;Decode string forwarded from BT receiver
;--------------------------------------------------------------------------------------------------------	
			
		movlw	.80
		movwf	FSR0L
		movlw	"P"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX3		;Not the string we want, exit
		incf	FSR0L,f
		movlw	"S"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX2		;Not the string we want, exit
		incf	FSR0L,f
		movlw	"U"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f	
		movlw	","
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f
		movlw	"0"
		xorwf	INDF0,w
		btfsc	STATUS,Z		
		incf	FSR0L,f	
		
		;At this point it's valid but what type?
		
		movlw	"1"
		xorwf	INDF0,w
		btfsc	STATUS,Z
		goto	MoveDetect		;Movement detected, flag it
		
		movlw	"9"
		xorwf	INDF0,w
		btfsc	STATUS,Z
		goto	EEPanic			;EEPROM related, panic time
		
		movlw	"2"
		xorwf	INDF0,w
		btfsc	STATUS,Z		
		goto	LEDpacketT		;LED packets
		
		movlw	"3"
		xorwf	INDF0,w
		btfsc	STATUS,Z		
		goto	BITpacket		;Packetbits packets
		
		movlw	"4"
		xorwf	INDF0,w
		btfsc	STATUS,Z		
		goto	PROpacket		;Profile packet
		
		movlw	"6"
		xorwf	INDF0,w
		btfsc	STATUS,Z		
		goto	Timepacket		;Time/alarm packets
		
		movlw	"7"
		xorwf	INDF0,w
		btfsc	STATUS,Z		
		goto	Relaypacket		;6x relay bytes packet

		movlw	"8"
		xorwf	INDF0,w
		btfsc	STATUS,Z		
		goto	Catpacket		;Remap cathodes
		
		bcf	INTflags,2		;Clear eeprom receive mode				
		goto	GPSdecodeX		;EXIT
		
GPSdecodeX2					;Starts with 'P' but not PSU						
		movlw	"R"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f
		movlw	"O"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		goto	PROpacket		;Profile packet
		
GPSdecodeX3					;Not 'P', try BIT						
		movlw	"B"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX4		;Not the string we want, exit
		incf	FSR0L,f		
		movlw	"I"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f	
		movlw	"T"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		goto	BITpacket		;Packetbits packets	
		
GPSdecodeX4					;Not 'B', try LED						
		movlw	"L"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX5		;Not the string we want, exit
		incf	FSR0L,f		
		movlw	"E"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f	
		;movlw	"D"
		;xorwf	INDF0,w
		;btfss	STATUS,Z
		;goto	GPSdecodeX		;Not the string we want, exit
		goto	LEDpacketT		;LED packets	
		
GPSdecodeX5					;Not 'L', try ALA						
		movlw	"A"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX6		;Not the string we want, exit
		incf	FSR0L,f		
		movlw	"L"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f	
		movlw	"A"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		goto	Timepacket2ALA
		
GPSdecodeX6					;Not 'A', try DIM						
		movlw	"D"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f		
		movlw	"I"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f	
		movlw	"M"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		goto	Timepacket1DIM
			
;---------------------------		
Relaypacket	
		call	GPSgetbyte
		movwf	RelayD1
		call	GPSgetbyte
		movwf	RelayD2
		call	GPSgetbyte
		movwf	RelayD3
		call	GPSgetbyte
		movwf	RelayD4
		call	GPSgetbyte
		movwf	RelayD5
		call	GPSgetbyte
		movwf	RelayD6
		call	GPSgetbyte
		movwf	Relaycount
		incf	Relaycount,f
GPSdecodeXflash					;Exit and flash blue LED	
		movlw	.31
		movwf	LEDMB
		call	LEDon		
		goto	GPSdecodeX
		
;---------------------------	
		
BITpacket	
		call	GPSgetbyte		;Get bit 0-15
		andlw	.15
		movwf	VerK
		call	GPSgetbyte		;Get value
		btfsc	VerK,3			;>7? byte 2 if so
		goto	BITpacketII
		movf	PacketBits1,w
		movwf	VerO
		call	BITbash
		movf	VerO,w
		movwf	PacketBits1
		goto	BITpacketX
			
BITpacketII
		bcf	VerK,3
		movf	PacketBits2,w
		movwf	VerO
		call	BITbash
		movf	VerO,w
		movwf	PacketBits2
				
BITpacketX		
		BANKSEL	PORTA
		movlw	0x20
		movwf	FSR1H
		swapf	ProfileByte,w
		movwf	UTL
		lslf	UTL,w
		addlw	0x8c			;Point at 128+(profile*32)+0c
		movwf	FSR1L
		movf	PacketBits1,w
		movwf	INDF1			;Store packetbits1 in profile
		incf	FSR1L,f
		movf	PacketBits2,w
		movwf	INDF1			;Store packetbits2 in profile
		goto	GPSdecodeXflash
		
BITbash						;Modify bit x in packetbits y
		incf	VerK,f			;Set to +1
		decfsz	VerK,f
		goto	BITpacket1
		bcf	VerO,0
		btfsc	GPStemp,0		;Value 0 or 1?
		bsf	VerO,0
		return
BITpacket1		
		decfsz	VerK,f
		goto	BITpacket2
		bcf	VerO,1
		btfsc	GPStemp,0		;Value 0 or 1?
		bsf	VerO,1
		return	
BITpacket2		
		decfsz	VerK,f
		goto	BITpacket3
		bcf	VerO,2
		btfsc	GPStemp,0		;Value 0 or 1?
		bsf	VerO,2
		return		
BITpacket3		
		decfsz	VerK,f
		goto	BITpacket4
		bcf	VerO,3
		btfsc	GPStemp,0		;Value 0 or 1?
		bsf	VerO,3
		return
BITpacket4		
		decfsz	VerK,f
		goto	BITpacket5
		bcf	VerO,4
		btfsc	GPStemp,0		;Value 0 or 1?
		bsf	VerO,4
		return
BITpacket5		
		decfsz	VerK,f
		goto	BITpacket6
		bcf	VerO,5
		btfsc	GPStemp,0		;Value 0 or 1?
		bsf	VerO,5
		return
BITpacket6		
		decfsz	VerK,f
		goto	BITpacket7
		bcf	VerO,6
		btfsc	GPStemp,0		;Value 0 or 1?
		bsf	VerO,6
		return
BITpacket7		
		bcf	VerO,7
		btfsc	GPStemp,0		;Value 0 or 1?
		bsf	VerO,7
		return
		
		
PROpacket					;Select profile# 0-3
		call	GPSgetbyte
		andlw	.3
		movwf	ProfileByte
		call	ProfUpdate
		goto	GPSdecodeXflash		
	
Timepacket			
		call	GPSgetbyte		
		movlw	.1			;Load time?
		xorwf	GPStemp,w
		btfss	STATUS,Z		
		goto	Timepacket1		
		movlw	.31
		movwf	LEDMB
		call	LEDon
		call	GPSgetbyte
		movwf	GHouk
		call	GPSgetbyte
		movwf	GMink
		call	GPSgetbyte
		movwf	GSeck
		call	GPSgetbyte
		movwf	GDayk
		call	GPSgetbyte
		movwf	GMonk
		call	GPSgetbyte
		movwf	GYeak
		goto	IRtimeupd
			
Timepacket1		
		movlw	.2			;Load dim on/off times?
		xorwf	GPStemp,w
		btfss	STATUS,Z		
		goto	Timepacket2
Timepacket1DIM
		call	GPSgetbyte
		movwf	HDimOn
		call	GPSgetbyte
		movwf	MDimOn
		call	GPSgetbyte
		movwf	HDimOff
		call	GPSgetbyte
		movwf	MDimOff
		call	GPSgetbyte
		movwf	VerK
		
		movlw	0x20
		movwf	FSR1H
		swapf	ProfileByte,w
		movwf	UTL
		lslf	UTL,w
		addlw	0x9a			;Point at 128+(profile*32)+26
		movwf	FSR1L
		movf	HDimOn,w	
		movwf	INDF1			;backup dim on
		incf	FSR1L,f
		movf	MDimOn,w
		movwf	INDF1
		incf	FSR1L,f
		movf	HDimOff,w	
		movwf	INDF1			;backup dim off
		incf	FSR1L,f
		movf	MDimOff,w
		movwf	INDF1
		incf	FSR1L,f
		movf	VerK,w
		movwf	INDF1
		goto	GPSdecodeXflash	
		
Timepacket2		
		movlw	.3			;Load alarm and set?
		xorwf	GPStemp,w
		btfss	STATUS,Z		
		goto	Timepacket3
Timepacket2ALA
		call	GPSgetbyte
		movwf	Ahouk
		call	GPSgetbyte
		movwf	Amink
		bsf	PacketBits2,2
		movlw	0x20
		movwf	FSR1H
		swapf	ProfileByte,w
		movwf	UTL
		lslf	UTL,w
		addlw	0x8d			;Point at 128+(profile*32)+13
		movwf	FSR1L
		movf	PacketBits2,w	
		movwf	INDF1			;backup flags
		movlw	.11			;Point to alarm time in this profile
		addwf	FSR1L,f
		movf	Ahouk,w
		movwf	INDF1
		incf	FSR1L,f
		movf	Amink,w			;Backup alarm time
		movwf	INDF1
		goto	GPSdecodeXflash	
		
Timepacket3	
		movlw	.4			;Set timezone offset
		xorwf	GPStemp,w
		btfss	STATUS,Z		
		goto	Timepacket4
		call	GPSgetbyte
		movwf	Zonecorr
		movlw	.24
		xorwf	Zonecorr,w
		btfsc	STATUS,Z		;Hours more than 24?
		clrf	Zonecorr		
		goto	GPSdecodeXflash	
			
Timepacket4		
		goto	GPSdecodeX
		
;---------------------------		
LEDpacketT		
		BANKSEL	PORTA
		movlw	0x20
		movwf	FSR1H
		swapf	ProfileByte,w
		movwf	UTL
		lslf	UTL,w
		addlw	0x80			;Point at 128+(profile*32) - clock LED start
		movwf	FSR1L
				
		call	GPSgetbyte		;Get LED #
		decf	GPStemp,f		;Reduce by 1 so 0-9
		movlw	.15
		andwf	GPStemp,f
		movf	GPStemp,w
		movwf	VerK
		addwf	VerK,f
		addwf	VerK,f			;Multiply LED by 3 - 3 nybbles per LED
		call	IncAndSkip

		movlw	"G"			;Add 0 for green nybble
		xorwf	INDF0,w
		movlw	.2
		btfsc	STATUS,Z
		goto	LEDpacketII
		
		movlw	"R"			;Add 1 for red nybble
		xorwf	INDF0,w
		movlw	.1
		btfsc	STATUS,Z
		goto	LEDpacketII
		
		movlw	"B"			;Add 2 for blue nybble
		xorwf	INDF0,w
		movlw	.0
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not R,G or B, exit			
		
LEDpacketII	
		addwf	VerK,f
		btfss	GPStemp,3			;Is it >7?
		goto	LEDpacketclock
		movlw	.8			;LED in base?
		xorwf	GPStemp,w
		btfsc	STATUS,Z		
		goto	LEDpacketbase		;Goto base LED save 1
		movlw	.9			;LED in base?
		xorwf	GPStemp,w
		btfsc	STATUS,Z
		goto	LEDpacketbase
		goto	GPSdecodeX
	
LEDpacketbase
		movlw	.9			;Point to LED base locations
		addwf	FSR1L,f	
		
LEDpacketclock		
		call	GPSgetbyte
		movlw	.15
		andwf	GPStemp,f
		
		lsrf	VerK,w
		addwf	FSR1L,f			;Point to LED
		btfsc	VerK,0			;High or low nybble?
		goto	LEDpacketLow
		movlw	0xf
		andwf	INDF1,f			;Clear high nybble
		swapf	GPStemp,f
		goto	LEDpacketAdd
	
LEDpacketLow
		movlw	0xf0
		andwf	INDF1,f			;Clear low nybble	
		
LEDpacketAdd		
		movf	GPStemp,w
		addwf	INDF1,f
		call	ProfUpdate		;Load updated LEDs into working registers
		goto	GPSdecodeXflash
	
ProfUpdate					;Load profile in ProfileByte to working memory	
		
		movlw	0x20
		movwf	FSR1H
		swapf	ProfileByte,w
		movwf	UTL
		lslf	UTL,w
		addlw	0x8c			;Point at 128+(profile*32)+12
		movwf	FSR1L
		movf	INDF1,w
		movwf	PacketBits1		;Copy profile control bits to working regs
		incf	FSR1L,f
		movf	INDF1,w
		movwf	PacketBits2
		movlw	.8			;Point to LED base locations
		addwf	FSR1L,f
		
		movf	INDF1,w	
		movwf	GPStemp
		incf	FSR1L,f			;Copy profile base LEDs to working regs
		andlw	0x0f
		movwf	LEDRR
		swapf	GPStemp,w
		andlw	0x0f
		movwf	LEDRB
		
		movf	INDF1,w	
		movwf	GPStemp
		incf	FSR1L,f
		andlw	0x0f
		movwf	LEDLB
		swapf	GPStemp,w
		andlw	0x0f
		movwf	LEDRG
		
		movf	INDF1,w	
		movwf	GPStemp
		incf	FSR1L,f
		andlw	0x0f
		movwf	LEDLG
		swapf	GPStemp,w
		andlw	0x0f
		movwf	LEDLR
		bsf	MENUflags2,4		;Send # to displays next packet
		return
	
;---------------------------
		
Catpacket	
		call	GPSgetnybbled		;Cats 0,1
		andlw	0x0f
		movwf	Catty1
		swapf	GPStemp,w
		andlw	0x0f
		movwf	Catty0
		call	GPSgetnybbled		;Cats 2,3
		andlw	0x0f
		movwf	Catty3
		swapf	GPStemp,w
		andlw	0x0f
		movwf	Catty2
		call	GPSgetnybbled		;Cats 4,5
		andlw	0x0f
		movwf	Catty5
		swapf	GPStemp,w
		andlw	0x0f
		movwf	Catty4
		call	GPSgetnybbled		;Cats 6,7
		andlw	0x0f
		movwf	Catty7
		swapf	GPStemp,w
		andlw	0x0f
		movwf	Catty6
		call	GPSgetnybbled		;Cats 8,9
		andlw	0x0f
		movwf	Catty9
		swapf	GPStemp,w
		andlw	0x0f
		movwf	Catty8
			
		goto	GPSdecodeXflash	
		
;---------------------------
		
MoveDetect	;done
		movlw	.200			;Countdown in seconds
		movwf	Movecount
		goto	GPSdecodeX
		
;---------------------------		
;EEPanic;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;NOT DETECTING STRING ?;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		incf	FSR0L,f
		movlw	"E"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f
		movlw	"P"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		incf	FSR0L,f
		movlw	"M"
		xorwf	INDF0,w
		btfss	STATUS,Z
		goto	GPSdecodeX		;Not the string we want, exit
		
EEPanic		
		bsf	INTflags,2		;Set eeprom receive mode
		BANKSEL	PORTA
		bcf	INTflags,7		;Clear check if GPS data received	
		movlw	.4 			;Indicate EEPROM mode activated
		call	ProfEELoad	
		movlw	.6
		movwf	Tcount			;Extend timeout count
		
		clrf	UTM
		clrf	UTL			;EEPROM pointers
	
FirmLoop1					;Byte 0000 for checksum
		btfss	INTflags,2		;Check if timed out yet
		goto	FirmExitFail
		btfss	INTflags,1		;Check if byte received yet
		goto	FirmLoop1
		
		
		movlw	0x20			;Remove leading spaces
		xorwf	GPStemp1,w
		btfss	STATUS,Z
		goto	FirmLoop1a
		
		bcf	INTflags,1
		goto	FirmLoop1	
		
		
		
FirmLoop1a		
		movf	GPStemp1,w		;get rxd
		movwf	EEsumL			;First byte of image,save as checksum
		movwf	IICC
		bcf	INTflags,1
		movlw	'O'
		movwf	VerO
		movlw	'K'
		movwf	VerK
		call	IICwrite
		incf	UTL,f		
FirmLoop2					;Byte 0001
		btfss	INTflags,2		;Check if timed out yet
		goto	FirmExitFail
		btfss	INTflags,1		;Check if byte received yet
		goto	FirmLoop2
		movf	GPStemp1,w		;get rxd
		movwf	EEsumH			;Second byte of image,save as checksum
		movwf	IICC
		bcf	INTflags,1
		movlw	'O'
		movwf	VerO
		movlw	'K'
		movwf	VerK
		call	IICwrite
		incf	UTL,f		
			
FirmLoop					;0002-0fff saved here
		btfss	INTflags,2		;Check if timed out yet
		goto	FirmExitFail
		btfss	INTflags,1		;Check if byte received yet
		goto	FirmLoop		
		movf	GPStemp1,w		;get rxd
		movwf	IICC
		
		bcf	INTflags,1
		movlw	'O'
		movwf	VerO
		movlw	'K'
		movwf	VerK
		call	IICwrite	
		movlw	.2
		movwf	Tcount			;Extend timeout count

		incf	UTL,f				
		incf	UTL,f
		decfsz	UTL,f
		goto	FirmLoop
		incf	UTM,f	
		movlw	0x10			;Up to 0x1000 yet?
		xorwf	UTM,w
		btfss	STATUS,Z			
		goto	FirmLoop
		BANKSEL	PORTA
		movlw	.5  			;Indicate EEPROM mode stage2 - received OK
		call	ProfEELoad

		movlw	0x10
		movwf	CalcsumH
		movlw	0x28
		movwf	CalcsumL		;Preload with first 2 bytes of ROM
		clrf	UTM			;Now verify the saved image is OK
		clrf	UTL			;EEPROM pointers
		incf	UTL,f
		incf	UTL,f
		
FirmVLoop
		call	IICread
		movf	IICC,w
		bcf	STATUS,C		;Add bytes to checksum
		btfss	UTL,0			;Is address even? add to CalcsumL
		addwf	CalcsumL,f
		btfsc	STATUS,C
		incf	CalcsumH,f
		btfsc	UTL,0			;Is address odd? add to CalcsumH
		addwf	CalcsumH,f
		incf	UTL,f
		incf	UTL,f
		decfsz	UTL,f
		goto	FirmVLoop
		incf	UTM,f	
		movlw	.2
		movwf	Tcount			;Extend timeout count
	
		movlw	0x10			;Up to 0x1000 yet?
		xorwf	UTM,w
		btfss	STATUS,Z				
		goto	FirmVLoop

		movf	CalcsumH,w		;Compare checksum bytes
		subwf	EEsumH,f
		incf	EEsumH,f
		decfsz	EEsumH,f
		goto	FirmExitFail2		;Checksum fail - saved data mismatch
		movf	CalcsumL,w
		subwf	EEsumL,f
		incf	EEsumL,f
		decfsz	EEsumL,f
		goto	FirmExitFail2		;Checksum fail	
		movlw	.6  			;Indicate EEPROM mode stage3 - checksum OK
		call	ProfEELoad
	
		
		clrf	UTM
		movlw	.5			;Now ID the eeprom type and store in EEID
		movwf	UTL
		call	IICread	
		movf	IICC,w
		movwf	EEID1
		movlw	.7			;Now ID the eeprom type and store in EEID
		movwf	UTL
		call	IICread	
		movf	IICC,w
		movwf	EEID2
		
		movlw	"B"
		xorwf	EEID1,w
		btfss	STATUS,Z
		goto	FirmExitFail3	
		movlw	"A"
		xorwf	EEID2,w
		btfss	STATUS,Z
		goto	FirmExitFail3
		goto	EEnewrestore		;update eeprom and reset
				
FirmExitFail3		
		movlw	.7  			;Indicate EEPROM failed - ID
		call	ProfEELoad
		movlw	.7 
		goto	FirmExitZ	
		
FirmExitFail2		
		movlw	.8  			;Indicate EEPROM failed - checksum
		call	ProfEELoad
		movlw	.8 
		goto	FirmExitZ
FirmExitFail		
		movlw	.9  			;Indicate EEPROM failed - timeout
		call	ProfEELoad
		movlw	.9
FirmExitZ		
		call	ProfEELoad
		
FirmExit	
		bcf	INTflags,2
		clrf	Tcount
		BANKSEL	BAUDCON	
		BSF	RCSTA,4			;Switch on serial port
		BANKSEL	PORTA
		goto	GPSdecodeX
		
;--------------------------------------------------------------------------------------------------------
CorrectForZone					;Apply timezone correction to GPS time
;--------------------------------------------------------------------------------------------------------

		movf	Zonecorr,w
		addwf	GHouk,f
		movlw	.24
		subwf	GHouk,w
		btfss	STATUS,C		;Hours more than 24?
		goto	CorrectForZone2	;no rollover, exit
		movwf	GHouk

;	Increment day
	
		movf	GMonk,w
		movwf	Timetemp2
		movf	GYeak,w
		movwf	Timetemp3
		call	monthcode		;;Lookup table for days in each month Timetemp2=M Timetemp3=Y
		incf	GDayk,f
		xorwf	GDayk,w
		btfss	STATUS,Z
		goto	CorrectForZone2	;exit

;	Increment month

		clrf	GDayk
		incf	GDayk,f			;Set to the 1st
		incf	GMonk,f
		movlw	.13
		xorwf	GDayk,w
		btfss	STATUS,Z
		goto	CorrectForZone2	;exit
		clrf	GMonk
		incf	GMonk,f
;	increment_year	
		incf	GYeak,f

CorrectForZone2

		movf	Houk,w		;update 12/24 hours register
		movwf	Houk12
		movlw	.13
		subwf	Houk12,w
		btfss	STATUS,0
		goto 	VeriTime1		
		movwf	Houk12
		incf	Houk12,f

		return

;--------------------------------------------------------------------------------------------------------

CorrectForZone5
		movf	Timetemp2,w
		subwf	GHouk,f
		btfsc	STATUS,C		;Rollover?
		goto	CorrectForZone2	;exit
		movlw	.24
		addwf	GHouk,f			;Complement 24h, decrement date counters
		decfsz	GDayk,f
		goto	CorrectForZone2	;exit

		decfsz	GMonk,f
		goto	CorrectForDays
		movlw	.12
		movwf	GMonk
		decf	GYeak,f
CorrectForDays					;Load correct no. of days for current month
		movf	GYeak,w
		movwf	Timetemp3		;temp year for leap year test	
		movf	GMonk,w
		movwf	Timetemp2
		call	monthcode		;;Lookup table for days in each month Timetemp2=M Timetemp3=Y
		movwf	GDayk
		decf	GDayk,f
		goto	CorrectForZone2	;exit

;--------------------------------------------------------------------------------------------------------
CorrectForBST					;Apply summertimecorrection to LIVE time
;--------------------------------------------------------------------------------------------------------

		btfss	PacketBits2,5		;Add or subtract?
		goto	CorrectForBST5
		incf	Houk,f
		movlw	.24
		xorwf	Houk,w
		btfss	STATUS,Z		;Hours more than 24?
		goto	CorrectForBST2		;no rollover, exit
		clrf	Houk

;	Increment day
	
		movf	Monk,w
		movwf	Timetemp2
		movf	Yeak,w
		movwf	Timetemp3
		call	monthcode		;;Lookup table for days in each month Timetemp2=M Timetemp3=Y
		incf	Dayk,f
		xorwf	Dayk,w
		btfss	STATUS,Z
		goto	CorrectForBST2	;exit

;	Increment month

		clrf	Dayk
		incf	Dayk,f			;Set to the 1st
		incf	Monk,f
		movlw	.13
		xorwf	Monk,w
		btfss	STATUS,Z
		goto	CorrectForBST2	;exit
		clrf	Monk
		incf	Monk,f
;	increment_year	
		incf	Yeak,f

CorrectForBST2
		movf	Houk,w			;update 12/24 hours	register
		movwf	Houk12
		movlw	.13
		subwf	Houk12,w
		btfss	STATUS,0
		goto 	VeriTime1		
		movwf	Houk12
		incf	Houk12,f

		return

;--------------------------------------------------------------------------------------------------------

CorrectForBST5
		incf	Houk,f
		decfsz	Houk,f
		goto	CorrectForBST2A
		movlw	.23
		movwf	Houk			;Set to 23xx
		decfsz	Dayk,f
		goto	CorrectForBST2

		decfsz	Monk,f
		goto	CorrectForDaysBST
		movlw	.12
		movwf	Monk
		decf	Yeak,f
CorrectForDaysBST				;Load correct no. of days for current month
		movf	Yeak,w
		movwf	Timetemp3		;temp year for leap year test	
		movf	Monk,w
		movwf	Timetemp2
		call	monthcode		;Lookup table for days in each month Timetemp2=M Timetemp3=Y
		movwf	Dayk
		decf	Dayk,f
		goto	CorrectForBST2

CorrectForBST2A
		decf	Houk,f
		goto	CorrectForBST2

;--------------------------------------------------------------------------------------------------------
VeriTime						;Verify time is valid and update registers
;--------------------------------------------------------------------------------------------------------

		bcf	INTflags,5		;clear PM bit
		clrf	LEDMG			;LED indicator of PM
		movf	Houk,w			;update 12/24 register
		movwf	Houk12
		movlw	.13
		subwf	Houk12,w
		btfss	STATUS,0
		goto 	VeriTime1		
		movwf	Houk12
		incf	Houk12,f
		bsf	INTflags,5		;Set PM bit
		movlw	.2
		movwf	LEDMG			;LED indicator of PM


VeriTime1

		return
		
;--------------------------------------------------------------------------------------------------------
Dimmtest1						;Compare alarm and dim times and set flags
;--------------------------------------------------------------------------------------------------------

		movf	HDimOn,w
		xorwf	Houk,w
		btfss	STATUS,Z
		goto	Dimmtesttexit1		;not equal, quit
		movf	MDimOn,w
		xorwf	Mink,w
		btfss	STATUS,Z
		goto	Dimmtesttexit1		;not equal, quit
		bsf	PacketBits1,4		;Dim is on
		call	CallSavePB1

Dimmtesttexit1			

		movf	HDimOff,w
		xorwf	Houk,w
		btfss	STATUS,Z
		goto	alarmtest1
		movf	MDimOff,w
		xorwf	Mink,w
		btfss	STATUS,Z
		goto	alarmtest1
		bcf	PacketBits1,4		;Dim is off
		call	CallSavePB1
		
alarmtest1					;alarm code here
		btfss	PacketBits2,2
		clrf	LEDMG			;LED indicator of alarm set
		btfss	PacketBits2,2		;Is alarm enabled?
		return
		movlw	.2
		movwf	LEDMG
		movf	Ahouk,w
		xorwf	Houk,w
		btfss	STATUS,Z
		return
		movf	Amink,w
		xorwf	Mink,w
		btfss	STATUS,Z
		return
		bsf	INTflags,4		;Alarm triggered, will need to be switched off by IR
		return
		
		
		end

		
		
		
		
		
		
;Each profile block is 32 bytes as follows:
		
;PacketR1G1		equ 	0x00	;LED 4-bit data 1 Red, 1 Green			
;PacketB1R2		equ 	0x01	;LED 4-bit data 1 Blue, 2 Red	
;PacketG2B2		equ 	0x02	;LED 4-bit data 2 Green, 2 Blue
;PacketR3G3		equ 	0x03	;LED 4-bit data 3 Red, 3 Green
;PacketB3R4		equ 	0x04	;LED 4-bit data 3 Blue, 4 Red
;PacketG4B4		equ 	0x05	;LED 4-bit data 4 Green, 4 Blue
;PacketR5G5		equ 	0x06	;LED 4-bit data 5 Red, 5 Green			
;PacketB5R6		equ 	0x07	;LED 4-bit data 5 Blue, 6 Red			
;PacketG6B6		equ 	0x08	;LED 4-bit data 6 Green, 6 Blue				
;PacketR7G7		equ 	0x09	;LED 4-bit data 7 Red, 7 Green	
;PacketB7R8		equ 	0x0a	;LED 4-bit data 7 Blue, 8 Red
;PacketG8B8		equ 	0x0b	;LED 4-bit data 8 Green, 8 Blue
		
		
					;If bits 7,6,5 are 0 just display raw 'relay' data
;PacketBits1		equ 	0x0c	;Sent to flock, display mode control flags
					;Bit7 - Show Seconds - ignored on 6 digit displays
					;Bit6 - Show Date - use date colon/indicators
					;Bit5 - Show Time - Use internal timebase as backup
					;Bit4 - Cathode cleaning triggered - REPLACE WITH DIM DISPLAY
					;Bit3 - Fade transition off
					;Bit2 - Ripple on
					;Bit1 - Ripple direction
					;Bit0 - Ripple speed				
					
;PacketBits2		equ 	0x0d	;Not sent to flock, base control flags
					;Bit7 - Display off unswitched*
					;Bit6 - Display off switched movement sensor*
					;Bit5 - TOGGLE SUMMERTIME HOUR - adds or removes one hour*
					;Bit4 - D/M or M/D TOGGLE*
					;Bit3 - Sticky date/time mode*
					;Bit2 - Random length seconds*
					;Bit1 - Seconds tick*
					;Bit0 - 12/24 hour mode*
					
;PacketBits3		equ 	0x0e	;Not sent to flock, base control flags
					;Bit7 - 
					;Bit6 - 
					;Bit5 - 
					;Bit4 - 
					;Bit3 - 
					;Bit2 - 
					;Bit1 - Dim on/off*
					;Bit0 - Alarm on/off*
					
;Unused				0x0f	
					
;Unused				0x10	Future feature creep			
;Unused				0x11	Future feature creep
;Unused				0x12	Future feature creep			
;Unused				0x13	Future feature creep
;Unused				0x14	Future feature creep			
					
;BaseR1G1		equ 	0x15	;LED 4-bit data 1 Red, 1 Green for base		
;BaseB1R2		equ 	0x16	;LED 4-bit data 1 Blue, 2 Red	for base
;BaseG2B2		equ 	0x17	;LED 4-bit data 2 Green, 2 Blue	for base
					
;Unused				0x18	Alarm HH
;Unused				0x19	Alarm MM
;Unused				0x1a	Dim on time HH
;Unused				0x1b	Dim on time MM
;Unused				0x1c	Dim off time HH
;Unused				0x1d	Dim off time MM
;Unused				0x1e	;Dim brightness 0-f
;Unused				0x1f	Future feature creep